@@ -302,37 +302,62 @@ static PyObject * get_recent_state(dde_integrator const * const self, PyObject *
 	return (PyObject *) result;
 }
 
+npy_intp dim[1] = { {{n}} };
+
+PyObject * n_dim_read_only_array_from_data(void * data) {
+	PyObject * result = PyArray_SimpleNewFromData( 1, dim, TYPE_INDEX, data );
+	PyArray_CLEARFLAGS( (PyArrayObject *) result, NPY_ARRAY_WRITEABLE );
+	return result;
+}
+
 static PyObject * get_current_state(dde_integrator const * const self)
 {
 	assert(self->last_anchor);
-	
-	npy_intp dims[1] = { {{n}} };
-	PyArrayObject * result = (PyArrayObject *)PyArray_SimpleNew(1, dims, TYPE_INDEX);
-	
-	for (int index=0; index<{{n}}; index++)
-		* (double *) PyArray_GETPTR1(result, index) = self->last_anchor->state[index];
-	
-	return (PyObject *) result;
+	return n_dim_read_only_array_from_data(self->last_anchor->state);
 }
 
 static PyObject * get_full_state(dde_integrator const * const self)
 {
 	PyObject * py_past = PyList_New(0);
-    npy_intp dim[1] = { {{n}} };
 	for (anchor * ca = self->first_anchor; ca; ca = ca->next)
 		PyList_Append(
 			py_past,
 			PyTuple_Pack(
-				3,
-				PyFloat_FromDouble(ca->time),
-				PyArray_SimpleNewFromData(1, dim, TYPE_INDEX, ca->state),
-				PyArray_SimpleNewFromData(1, dim, TYPE_INDEX, ca->diff ) 
+					3,
+					PyFloat_FromDouble(ca->time),
+					n_dim_read_only_array_from_data(ca->state),
+					n_dim_read_only_array_from_data(ca->diff )
 				)
 			);
 	return py_past;
 }
 
 
+{% if callbacks|length %}
+static inline double callback(PyObject * Python_function, PyObject * arglist)
+{
+	PyObject * py_result = PyObject_CallObject(Python_function,arglist);
+	Py_DECREF(arglist);
+	double result = PyFloat_AsDouble(py_result);
+	Py_DECREF(py_result);
+	return result;
+}
+{% endif %}
+
+{% for function,nargs in callbacks %}
+static PyObject * callback_{{function}};
+# define {{function}}(...) callback(\
+		callback_{{function}}, \
+		Py_BuildValue( \
+				{% if nargs -%}
+				"(O{{'d'*nargs}})", n_dim_read_only_array_from_data(y) , __VA_ARGS__ \
+				{% else -%}
+				"(O)", n_dim_read_only_array_from_data(y) \
+				{% endif -%}
+			))
+{% endfor %}
+
+
 # define set_dy(i, value) (dY[i] = value)
 # define current_y(i) (y[i])
 # define past_y(t, i, anchor) (get_past_value(t, i, anchor))
@@ -564,12 +589,27 @@ static int initiate_past_from_list(dde_integrator * const self, PyObject * const
 static int dde_integrator_init(dde_integrator * self, PyObject * args)
 {
 	PyObject * past;
-	if (!PyArg_ParseTuple(args, "O!", &PyList_Type, &past))
+	if (!PyArg_ParseTuple(
+				args,
+				"O!{{'O'*callbacks|length}}",
+				&PyList_Type, &past
+				{% for function,nargs in callbacks %}
+				, &callback_{{function}}
+				{% endfor %}
+		))
 	{
 		PyErr_SetString(PyExc_ValueError,"Wrong input.");
 		return -1;
 	}
 	
+	{% for function,nargs in callbacks %}
+	if (!PyCallable_Check(callback_{{function}}))
+	{
+		PyErr_SetString(PyExc_TypeError,"Callback must be callable.");
+		return -1;
+	}
+	{% endfor %}
+	
 	self->first_anchor = NULL;
 	self->last_anchor = NULL;
 	
